/*
* AtachFile.c
*
* Copyright (C) 2001 by Adobe Systems Incorporated.
* All rights reserved.

  Requires write permission on the directory where the attachment
  is saved.
*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef SOLARIS
#include <thread.h>
#endif

#include "FdfTk.h"


/*
  These definitions are needed only on Unix, for multithreading
*/
#ifdef SOLARIS
#define INITIALIZE_FDF_THREADSAFE { \
   static mutex_t mutexObj; \
   mutex_init( &mutexObj, NULL, NULL ); \
   FDFRegisterThreadsafeCallbacks( \
      (ThreadsafeCallback)mutex_lock, \
      (ThreadsafeCallback)mutex_unlock, \
      (ThreadsafeCallback)mutex_destroy, &mutexObj ); }
#endif

#ifndef INITIALIZE_FDF_THREADSAFE
#define INITIALIZE_FDF_THREADSAFE
#endif


/*
  The error handling strategy we use in this example is to return to the client
  an indication of which call into the FDF API failed, and what the error code
  was. This is probably a good approach during the debugging phase, but not for
  production systems!
*/
static void HandleError(FDFErc theErc, const char* funcName, const char* parameter, FDFDoc inFDF)
{
    if (theErc != FDFErcOK) {
		if (inFDF) FDFClose(inFDF);
		printf("Content-type: text/plain\n\n");
		printf("Error %d in call to %s. Parameter: %s\n", theErc, funcName, parameter);

#ifdef UNIX_ENV /* Do not need to do this on Win32 */
		FDFFinalize();
#endif

		exit(1);
    }
}


void main(int argc, char *argv[], char *envp[])
{
    FDFDoc inFDF = NULL;
    FDFErc theErc;
    char cFileName[255] = "tmp\\";
    char cMIME[255];
    FILE* f;
    ASBool bKeepReading = true;
    ASInt32 howMany = atoi(getenv("CONTENT_LENGTH"));
    struct stat statBuf;

#ifdef UNIX_ENV
    /*
      On Unix only, need to define the platform-dependent functions used for
      threading.
      This particular example does not use threading, but we still wanted
      to show how the FDFRegisterThreadsafeCallbacks() API is used.
    */
    INITIALIZE_FDF_THREADSAFE;
    /*
      This example assumes a cgi-bin environment, where each invocation of
      the program spawns a new process. In a multi-threaded environment,
      FDFInitialize() should only be called once, by the first thread in the
      application. Once again, this is Unix only. On Win32 the initialization
      and multithreading stuff is taken care of automatically by the FdfTk.dll
    */
    FDFInitialize();
#endif

    theErc = FDFOpen("-", howMany, &inFDF);
    HandleError(theErc, "FDFOpen", getenv("CONTENT_LENGTH"), inFDF);
    theErc = FDFExtractAttachment(inFDF, "filename", cFileName, sizeof(cFileName), true/*bIsFilePath*/,
				  cMIME, sizeof(cMIME));
    HandleError(theErc, "FDFExtractAttachment", "filename", inFDF);
    FDFClose(inFDF);

    stat(cFileName, &statBuf);
    printf("Content-Type: %s\n", cMIME);
    printf("Content-Length: %d\n\n", statBuf.st_size);
    fflush(stdout); /* Emit the HTTP header now! */

    f = fopen(cFileName, "rb");

    while (bKeepReading) {
	char cBuf[8192];
      	size_t nRead = fread(cBuf, sizeof(char), sizeof(cBuf), f);

	if (nRead)
	    fwrite(cBuf, sizeof(char), nRead, stdout);

	bKeepReading = (nRead == sizeof(cBuf));
    }

    fclose(f);
    remove(cFileName);

#ifdef UNIX_ENV
    /*
      This example assumes a cgi-bin environment, where each invocation of
      the program spawns a new process. In a multi-threaded environment,
      FDFFinalize() should only be called once, by the last thread in the
      application. Once again, this is Unix only. On Win32 the initialization
      and multithreading stuff is taken care of automatically by the FdfTk.dll
    */
    FDFFinalize();
#endif

    exit(0);
}
