/************************************************************************
Catalog.cpp
 
ADOBE SYSTEMS INCORPORATED
Copyright [1999] - [2001] Adobe Systems Incorporated
All Rights Reserved

NOTICE:  Adobe permits you to use, modify, and distribute 
this file in accordance with the terms of the Adobe license 
agreement accompanying it.  If you have received this file 
from a source other than Adobe, then your use, modification, 
or distribution of it requires the prior written permission of Adobe.

Use:
	1) Compile the project using VC++ 6.0.

	2) Install the Catalog.cgi into a directory with execute permissions
		for your server. These samples for Windows 2000 assume you have
		given execute permissions to your \inetpub\Scripts\ directory.

		You should verify that the submit URL used for the Submit button in
		CatEnglish.pdf and/or CatFrench.pdf point at the cgi file.

	3) Install the Server PDF files in the path 'PDFs\Catalog' in
		the root directory, e.g.:
			c:\inetpub\wwwroot\PDFs\Catalog\xxx.pdf
		To change this location you need to modify the path used in the
		FDFSetFile call below.
	
	4) Install the CGI PDF files into the same directory as the cgi file.
		You can change this location by modifying the paths in the
		ProdPicFileName structure.

	5) In your browser, open the CatEnglish.pdf or CatFrench.pdf file. 
		Choose your options and then click the Submit button.


**************************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#ifdef SOLARIS
#include <thread.h>
#endif

#include "FdfTk.h"


/*
  These definitions are needed only on Unix, for multithreading
*/
#ifdef SOLARIS
#define INITIALIZE_FDF_THREADSAFE { \
   static mutex_t mutexObj; \
   mutex_init( &mutexObj, NULL, NULL ); \
   FDFRegisterThreadsafeCallbacks( \
      (ThreadsafeCallback)mutex_lock, \
      (ThreadsafeCallback)mutex_unlock, \
      (ThreadsafeCallback)mutex_destroy, &mutexObj ); }
#endif

#ifndef INITIALIZE_FDF_THREADSAFE
#define INITIALIZE_FDF_THREADSAFE
#endif



/*
  This sample application starts with the user viewing CatEnglish.pdf or
  CatFrench.pdf. There are two checkboxes that allow the user to select a
  category of products they are interested in viewing. There is also a pair
  of radio buttons that can be used to select the country (US or Canada), 
  which governs the language of follow-up web pages, and the denomination (i.e.
  US vs. Canadian dollars). Based on the selections in this first web page, an
  FDF is constructed at the server, and sent back to the client. The FDF gives
  (as the value of the /F key) the name of the AcroForm that needs to be fetched
  in order to import the FDF data into it. There are four possibilities: 2 or 4
  page catalog, and French or English.
*/


#define NUM_PRODUCTS 7

/*
  Each of these files contains the image of one item in the catalog. FDF carries
  each image from the server to the AcroForm at the client, as the new
  "appearance" (i.e. the value of the /AP key) of a (non-interactive) button
  field in the AcroForm.
*/
static char* const ProdPicFileName [NUM_PRODUCTS] = {
    "clip.pdf",
    "clipboard.pdf",
    "folder.pdf",
    "folderexpand.pdf",
    "folderopen.pdf",
    "stapler.pdf",
    "tapedispenser.pdf"
};
  
/*
  Product names (in English or French), to be used as the value of fields
  called "name1", "name2", etc.
*/
static char* const ProdName [][NUM_PRODUCTS] = {
    {
	"BINDER CLIPS",
	"CLIPBOARD",
	"FILE FOLDER",
	"EXPANDING FILES",
	"OPEN TOP FILES",
	"STAPLER",
	"TAPE DISPENSER"
    },
    {
	"ATTACHES POUR CARTABLE",
	"PRESSE-PAPIER",
	"DOSSIER",
	"DOSSIER EXTENSIBLES",
	"OUVRIR PREMIERS FICHIERS",
	"BROCHEUSE",
	"ALIMENTATEUR DE RUBAN"
    }
};
  
/*
  Product descriptions (in English or French), to be used as the value of fields
  called "desc1", "desc2", etc.
*/
static char* const ProdDesc [][NUM_PRODUCTS] = {
    {
	"Handles can be hung, folded flat against clipped material, or removed for permanent binding.",
	"Hardboard.",
	"11pt. long grained fibers for strength.",
	"With flap and elastic cord. 7/8 inch capacity pockets. Reinforced double-thick front and black.",
	"25 pockets with two indexing systems",
	"Holds 200 standard staples",
	"No moving parts"
    },
    {
	"Poignee accrochables pliable et amovible si desire.",
	"Tableau.",
	"11pt. fibres rigides.",
	"Avec languette et elastique. Capacite de 3cm par pochette. Double epaisseur avant et arriere.",
	"25 sachets avec deux systemes d'indexation",
	"Contient 200 broches",
	"Sans piece amovible"
    }
};
  
/*
  Product numbers, to be used as the value of fields called "num1", "num2", etc.
*/
static char* const ProdNum [NUM_PRODUCTS] = {
    "BC-1020",
    "HC-10A",
    "WJ-100-33",
    "WJ-102-33",
    "WJ-101-33",
    "SS-2001",
    "TD-347"
};
  
/*
  Product prices (in US dollars), to be used as the value of fields called
  "price1", "price2", etc. These need to get multiplied by a currency factor
  to convert to Canadian dollars.
*/
static double const ProdPrice [NUM_PRODUCTS] = {
    3.0,
    1.9,
    12.5,
    10.59,
    11.77,
    10.0,
    20.0
};
  
/*
  Some items are categorized as "desktop" and some as "folders", and the user
  selects on the initial AcroForm which category he wants to view.
*/
static ASBool const IsDesktop [NUM_PRODUCTS] = {
    true,
    true,
    false,
    false,
    false,
    true,
    true
};

/*
  Error message if the user doesn't select a category. This is carried by the
  FDF as the value of the /Status key, which causes an alert to pop-up at the
  client. This is not the only way to convey status. We could also use a text
  field for this purpose, possibly one that is normally hidden and becomes
  visible only if needed. Visibility of a field can be controlled via flags
  changed through FDF, e.g. FDFSetFlags(theFDF, "status field", FDFClrF, 2).
*/
static char* const SelectCategoryPlease [] = {
    "Please select \"Desktop Supplies\" and/or \"Folders\"",
    "Fourniture de bureau ou dossiers?"
};


/*
  The error handling strategy we use in this example is to return to the client
  an indication of which call into the FDF API failed, and what the error code
  was. This is probably a good approach during the debugging phase, but not for
  production systems!
*/
static void HandleError(FDFErc theErc, char* funcName, char* parameter,
			FDFDoc inFDF, FDFDoc outFDF)
{
    if (theErc != FDFErcOK) {

	if (inFDF) FDFClose(inFDF);
	if (outFDF) FDFClose(outFDF);
	printf("Content-type: text/plain\n\n");
        printf("Error %d in call to %s. Parameter: %s\n", theErc, funcName,
	       parameter);

#ifdef UNIX_ENV /* Do not need to do this on Win32 */
	FDFFinalize();
#endif
	exit(1);
    }
}


void main(int argc, char *argv[], char *envp[])
{
    FDFDoc inFDF = NULL;
    FDFDoc outFDF = NULL;
    FDFErc theErc;
    char cBuf[255];
    char cCountry[5];
    int iCountry = 0;
    double fCurrencyFactor = 1.0;
    ASBool bWantsDesktop = false;
    ASBool bWantsFolder = false;
    ASInt32 howMany = atoi(getenv("CONTENT_LENGTH"));
    int i;
    int nItems = 0;

#ifdef UNIX_ENV
    /*
      On Unix only, need to define the platform-dependent functions used for
      threading.
      This particular example does not use threading, but we still wanted
      to show how the FDFRegisterThreadsafeCallbacks() API is used.
    */
    INITIALIZE_FDF_THREADSAFE;
    /*
      This example assumes a cgi-bin environment, where each invocation of
      the program spawns a new process. In a multi-threaded environment,
      FDFInitialize() should only be called once, by the first thread in the
      application. Once again, this is Unix only. On Win32 the initialization
      and multithreading stuff is taken care of automatically by the FdfTk.dll
    */
    FDFInitialize();
#endif

    /*
      Open the FDF data submitted from the client. This example assumes cgi-bin,
      where the POSTed data is available at stdin, and its length is in
      environment variable CONTENT_LENGTH.
    */
    theErc = FDFOpen("-", howMany, &inFDF);
    HandleError(theErc, "FDFOpen", getenv("CONTENT_LENGTH"), inFDF, outFDF);

    /*
      Get the value of the radio button called "country". The AcroForm was
      defined so that one button exports the value "ca", and the other "us".
    */
    theErc = FDFGetValue(inFDF, "country", cCountry, sizeof(cCountry), &howMany);
    HandleError(theErc, "FDFGetValue", "country", inFDF, outFDF);

    if (strcmp(cCountry, "ca") == 0) {
	iCountry = 1;
	fCurrencyFactor = 1.354;
    }

    /*
      Get the value of the checkbox called "desktop". If it is checked, we get
      "on" as the value, else we get "off". We could've changed the export value
      of the "on" state to something else, but decided to use the default.
    */
    theErc = FDFGetValue(inFDF, "desktop", cBuf, sizeof(cBuf), &howMany);
    HandleError(theErc, "FDFGetValue", "desktop", inFDF, outFDF);

    if (strcmp(cBuf, "on") == 0)
	bWantsDesktop = true;

    /*
      Get the value of the checkbox called "folders".
    */
    theErc = FDFGetValue(inFDF, "folders", cBuf, sizeof(cBuf), &howMany);
    HandleError(theErc, "FDFGetValue", "folders", inFDF, outFDF);

    if (strcmp(cBuf, "on") == 0)
	bWantsFolder = true;

    /* Create a new FDF to send the response to the client */
    theErc = FDFCreate(&outFDF);
    HandleError(theErc, "FDFCreate", "none", inFDF, outFDF);

    if (!bWantsFolder && !bWantsDesktop) {
	/*
	  Send an error message if the user doesn't select a category. This is carried
	  by the FDF as the value of the /Status key, which causes an alert to pop-up
	  at the client.
	*/
	theErc = FDFSetStatus(outFDF, SelectCategoryPlease[iCountry]);
	HandleError(theErc, "FDFSetStatus", "none", inFDF, outFDF);
    }
    else {
	/*
	  Find the products that the user is interested in
	*/
	for (i = 0; i < NUM_PRODUCTS; i++) {
	    if ((IsDesktop[i] && bWantsDesktop) || (!IsDesktop[i] && bWantsFolder)) {
		char cFldName[10];
		nItems++;

		/* Set the description of the item */
		sprintf(cFldName, "desc%d", nItems);
		theErc = FDFSetValue(outFDF, cFldName, ProdDesc[iCountry][i], false);
		HandleError(theErc, "FDFSetValue", cFldName, inFDF, outFDF);

		/* Its name */
		sprintf(cFldName, "name%d", nItems);
		theErc = FDFSetValue(outFDF, cFldName, ProdName[iCountry][i], false);
		HandleError(theErc, "FDFSetValue", cFldName, inFDF, outFDF);

		/* Its catalog number */
		sprintf(cFldName, "num%d", nItems);
		theErc = FDFSetValue(outFDF, cFldName, ProdNum[i], false);
		HandleError(theErc, "FDFSetValue", cFldName, inFDF, outFDF);

		/* Its price in the right currency */
		sprintf(cFldName, "price%d", nItems);
		sprintf(cBuf, "$%.2f", ProdPrice[i] * fCurrencyFactor);
		theErc = FDFSetValue(outFDF, cFldName, cBuf, false);
		HandleError(theErc, "FDFSetValue", cFldName, inFDF, outFDF);

		/* And finally its picture, obtained from page 1 of a PDF file */
		sprintf(cFldName, "pic%d", nItems);
		theErc = FDFSetAP(outFDF, cFldName, FDFNormalAP, NULL, ProdPicFileName[i], 1);
		HandleError(theErc, "FDFSetAP", ProdPicFileName[i], inFDF, outFDF);
	    }
	}

	/*
	  The FDF gives (as the value of the /F key) the name of the AcroForm that
	  needs to be fetched in order to import the FDF data into it. There are
	  four possibilities: 2 or 4 page catalog, and French or English.
	  In my installation, the files cat2u2.pdf, cat2c2.pdf, cat2u4.pdf and
	  cat2c4.pdf reside in the same directory as CatEnglish.pdf and CatFrench.pdf,
	  and have read permission for the user id that the web server is running as.
	  
	*/
	sprintf(cBuf, "http://localhost/PDFs/catalog/cat2%c%d.pdf", cCountry[0],
		(bWantsDesktop && bWantsFolder) ? 4 : 2);
	theErc = FDFSetFile(outFDF, cBuf);
	HandleError(theErc, "FDFSetFile", cBuf, inFDF, outFDF);
    }

    /*
      Don't forget to emit the correct HTTP header for the MIME type.
      By the way, remember when setting the URL of a SubmitForm action, to end
      it in #FDF. For example, the URL used in CatEnglish.pdf and CatFrench.pdf
      to submit to this cgi-bin program is:
      http:/localhost/Scripts/catalog.cgi#FDF
    */
    printf("Content-type: application/vnd.fdf\n\n");
    fflush(stdout); /* Emit the HTTP header now! */

    FDFSave(outFDF, "-"); /* Now send the FDF to stdout */
 
    FDFClose(inFDF); /* Clean up after yourself! */
    FDFClose(outFDF);

#ifdef UNIX_ENV
    /*
      This example assumes a cgi-bin environment, where each invocation of
      the program spawns a new process. In a multi-threaded environment,
      FDFFinalize() should only be called once, by the last thread in the
      application. Once again, this is Unix only. On Win32 the initialization
      and multithreading stuff is taken care of automatically by the FdfTk.dll
    */
    FDFFinalize();
#endif

    exit(0);
}
