/*
PhoneSearch.cpp

ADOBE SYSTEMS INCORPORATED
Copyright [1999] - [2001] Adobe Systems Incorporated
All Rights Reserved

NOTICE:  Adobe permits you to use, modify, and distribute 
this file in accordance with the terms of the Adobe license 
agreement accompanying it.  If you have received this file 
from a source other than Adobe, then your use, modification, 
or distribution of it requires the prior written permission of Adobe.

   This sample application starts with the user viewing PhoneDirSearch.pdf. There is 
   a text field which allows the user to type in a query and submit it to a server.
   After the server receives the FDF and parses the query, it searches a text file with 
   data and builds a new FDF on the server based on the result of the search and 
   sends it back to the client. FDF uses JavaScript to generate the fields on the fly 
   and after fields are added it populates them with data.
*/

#include <stdlib.h>
#include <malloc.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#ifdef SOLARIS
#include <thread.h>
#endif

#include "FdfTk.h"

/*
  These definitions are needed only on Unix, for multithreading
*/
#ifdef SOLARIS
#define INITIALIZE_FDF_THREADSAFE { \
   static mutex_t mutexObj; \
   mutex_init( &mutexObj, NULL, NULL ); \
   FDFRegisterThreadsafeCallbacks( \
      (ThreadsafeCallback)mutex_lock, \
      (ThreadsafeCallback)mutex_unlock, \
      (ThreadsafeCallback)mutex_destroy, &mutexObj ); }
#endif

#ifndef INITIALIZE_FDF_THREADSAFE
#define INITIALIZE_FDF_THREADSAFE
#endif


/*
  The error handling strategy we use in this example is to return to the client
  an indication of which call into the FDF API failed, and what the error code
  was. This is probably a good approach during the debugging phase, but not for
  production systems!
*/

static void HandleError(FDFErc theErc, const char* funcName, const char* parameter,
			FDFDoc outFDF)
{
    if (theErc != FDFErcOK) {
	if (outFDF) FDFClose(outFDF);
	printf("Content-type: text/plain\n\n");
	printf("Error %d in call to %s. Parameter: %s\n", theErc, funcName, parameter);

#ifdef UNIX_ENV /* Do not need to do this on Win32 */
	FDFFinalize();
#endif

	exit(1);
    }
}


/*
Each record takes one row in the text file and has all those fields 
stored one after another and separated by tab character.('\t')

Example:
P	Fred Nisevich	REG	608-456-3333	nisevich	usa.CA-Quad.6	Q6-111		Supreme Ruler and Commander
*/
struct Employee
{
    char group[5];
    char fName[100];
    char lName[100];
    char status[100];
    char phone[100];
    char email[100];
    char location[100];
    char office_num[100];
    char blank[3];
    char job_title[150];
};

/* 
This function is used to parse the string containing one record into its member fields,
using '/t' as a delimeter.
*/
static void strParse(char* inString, Employee *outEmployee)
{
    memset(outEmployee, 0, sizeof(Employee));

    for(int index=0; index < 10; ++index) {
	int n = strcspn(inString, "\t");

	char* destPtr;
	int destSize;

	switch(index) {
	case 0:
	    destPtr = outEmployee->group; 
	    destSize = sizeof(outEmployee->group);
	    break;
	case 1:
	    destPtr = outEmployee->fName; 
	    destSize = sizeof(outEmployee->fName);
	    break;
	case 2:
	    destPtr = outEmployee->lName; 
	    destSize = sizeof(outEmployee->lName);
	    break;
	case 3:
	    destPtr = outEmployee->status; 
	    destSize = sizeof(outEmployee->status);
	    break;
	case 4:
	    destPtr = outEmployee->phone; 
	    destSize = sizeof(outEmployee->phone);
	    break;
	case 5:
	    destPtr = outEmployee->email; 
	    destSize = sizeof(outEmployee->email);
	    break;
	case 6:
	    destPtr = outEmployee->location; 
	    destSize = sizeof(outEmployee->location);
	    break;
	case 7:
	    destPtr = outEmployee->office_num; 
	    destSize = sizeof(outEmployee->office_num);
	    break;
	case 8:
	    destPtr = outEmployee->blank; 
	    destSize = sizeof(outEmployee->blank);
	    break;
	case 9:
	    destPtr = outEmployee->job_title; 
	    destSize = sizeof(outEmployee->job_title);
	    break;
	default:
	    return;
	}
		
	strncpy(destPtr, inString, destSize < n ? destSize : n);
	destPtr[destSize-1] = 0x00;

	if(n)
	    inString += n + 1;
	else if(inString[0] == '\t')
	    ++inString;
	else
	    break;
    }
}

static char* miCopyStr(const char *str)
{
    /* A variation on strcpy, this function allocates and copies. */
    if (!str)
	return (char *)NULL;
 
    char* p = (char*)malloc(strlen(str) * sizeof(char) + 1);

    strcpy(p, str);

    return p;
}


void main(int argc, char *argv[], char *envp[])
{
    
    FDFDoc inFDF = NULL;  
    FDFDoc outFDF = NULL;
    FDFErc theErc;
    ASInt32 howMany = atoi( getenv("CONTENT_LENGTH") );
    char cBuf[50];
    FILE *fPtr = NULL;
    char line[250];
    int count=0;
    Employee *empPtr;
    char buf[2900];
    char newName[10];
    char indx[6];
    int Flag = 0;
    char strEmail[1000];
	

#ifdef UNIX_ENV
    /*
      On Unix only, need to define the platform-dependent functions used for
      threading.
      This particular example does not use threading, but we still wanted
      to show how the FDFRegisterThreadsafeCallbacks() API is used.
    */
    INITIALIZE_FDF_THREADSAFE;
    /*
      This example assumes a cgi-bin environment, where each invocation of
      the program spawns a new process. In a multi-threaded environment,
      FDFInitialize() should only be called once, by the first thread in the
      application. Once again, this is Unix only. On Win32 the initialization
      and multithreading stuff is taken care of automatically by the FdfTk.dll
    */
    FDFInitialize();
#endif

    /*
      Open the FDF data submitted from the client. This example assumes cgi-bin,
      where the POSTed data is available at stdin, and its length is in
      environment variable CONTENT_LENGTH.
    */
    theErc = FDFOpen("-", howMany, &inFDF);
    HandleError(theErc, "FDFOpen", "none", inFDF);
	
    // Create a new FDF to send the response to the client
    theErc = FDFCreate(&outFDF);
    HandleError(theErc, "FDFCreate", "none", outFDF);

    //Extract submitted query.
    theErc = FDFGetValue(inFDF, "findit", cBuf, sizeof(cBuf), &howMany);
    
    //Raise error when empty string is submitted
    if( theErc==FDFErcFieldNotFound ) {
	theErc = FDFSetStatus(outFDF, "Empty string was submitted. Please try again." );
	HandleError(theErc, "FDFSetStatus", "none", outFDF);
    }
    else {
	/* Open "database" file for reading. It is assumed that db2.txt is in the same folder
	   as this cgi program, therefore only the file name is needed */
	fPtr = fopen("db2.txt", "r" );
	if( fPtr == NULL ) {
	    theErc = FDFSetStatus(outFDF, "Data file could not be opened!\n" );
	    HandleError(theErc, "FDFSetStatus", "none",outFDF);
	}	
	else {	
	    //convert submitted string to lower case
	    for(int j=0; *(j+cBuf) !='\0' ; j++)
		*(j+cBuf) = tolower(*(j+cBuf));
			
	    //read TXT file line by line until End Of File is found 
	    count=0;
	    while( !feof(fPtr) ) {
		fgets( line , sizeof(line), fPtr);
		*(line+strlen(line)-1 )='\0';
				
		//convert line of data from file into lower case
		char* lineCopy = miCopyStr( line );
		for(int j=0; *(j+line) !='\0' ; j++) {	
		    if(  *(j+line) >= 65 &&  *(j+line) <= 90 )
			*(j+line) = *(j+line) + 32; 
		    if( *(j+line) == '\t' )
			*(j+line) = ' ';
		}
				
		//if match is found 
		if( strstr( line, cBuf ) != NULL ) {
		    //Generate a new FDF
		    empPtr = new Employee; // create a new record
		    strParse( lineCopy , empPtr ); // initiate the record with the values; 
					
		    sprintf(indx, "%d", count);
					
		    strcpy(newName,"name1.");
		    char* FirstAndLast = new char[ strlen(empPtr->fName ) + strlen(empPtr->lName ) + 2 ];
		    strcpy(FirstAndLast,empPtr->fName);
		    strcat(FirstAndLast," ");
		    strcat(FirstAndLast,empPtr->lName );
		    theErc = FDFSetValue(outFDF, strcat(newName,indx), FirstAndLast , true);
		    HandleError(theErc, "FDFSetValue", newName, outFDF);
		    delete [] FirstAndLast;
									
		    strcpy(newName,"phone.");
		    theErc = FDFSetValue(outFDF, strcat(newName,indx), empPtr->phone , true);
		    HandleError(theErc, "FDFSetValue", newName, outFDF);
										
		    //Generate string which holds all e-mail addresses of found records.
		    //This string later is passed to a document level JavaScript function called
		    //'SetCaption()'
		    if( count == 0 )
			strcpy(strEmail,empPtr->email);
		    else
			strcat(strEmail,empPtr->email);
		    strcat( strEmail,",");

		    char tmp1[30];
		    strcpy(newName,"email.");
		    sprintf( tmp1 ,"app.mailMsg(true, \"%s@world.com\")",empPtr->email  );
		    theErc = FDFSetJavaScriptAction(outFDF,strcat(newName,indx), FDFUp, tmp1 );
		    HandleError(theErc, "FDFSetJavaScriptAction",newName, outFDF);					
					
		    /*
		      This is used to remove first part from the location portion of the record
		      Originally some 'location' parts of the records are stored in the 
		      following format : usa.CA-EastTower.07
		      If such record is found, the first part of the string 
		      including dash  is get removed and only tower + floor number are displayed
		    */
		    strcpy(newName,"location.");
		    char* ptr;
		    if( (ptr = strstr( empPtr->location , "-" )) != NULL ) //if dash is found 
			theErc = FDFSetValue(outFDF, strcat(newName,indx), ptr+1 , true);
		    else
			theErc = FDFSetValue(outFDF, strcat(newName,indx), empPtr->location  , true);
		    HandleError(theErc, "FDFSetValue", newName, outFDF);
					
		    strcpy(newName,"office_num.");
		    theErc = FDFSetValue(outFDF, strcat(newName,indx), empPtr->office_num, true);
		    HandleError(theErc, "FDFSetValue", newName, outFDF);
					
		    strcpy(newName,"job_title.");
		    theErc = FDFSetValue(outFDF, strcat(newName,indx), empPtr->job_title , true);
		    HandleError(theErc, "FDFSetValue", newName, outFDF);
										
		    Flag=1;
		    count++; // count how many records matching the query are found 
		    delete empPtr;
		}

		free(lineCopy);
	    }
		  
	    // set a value for field which displays how many records are found. 
	    char* res = new char[strlen("Total records found = ") + 5];
	    strcpy(res,"Total records found = ");
	    sprintf(indx, "%d", count);
	    strcat(res,indx);
	    theErc = FDFSetValue(outFDF, "result", res , true);
	    HandleError(theErc, "FDFSetValue", "result", outFDF);	
	    delete [] res;
		  
	    if( Flag ) {
		/*
		  Generate Document Level JavaScript function 'DrawOneRow()'
		  which is responsible for drawing one row of fields to fit 
		  one record of data
		*/
		strcpy(buf,"var i=0.2;\r"); 
		strcat(buf,"var p=0;\r");
		strcat(buf,"function DrawOneRow()\r");
		strcat(buf,"{\r");
		strcat(buf,"\tvar inch=72;\r\r");
		strcat(buf,"\tif( j%10 == 0 && j )\r");
		strcat(buf,"\t{\r");
			  
		strcat(buf,"\t\tthis.spawnPageFromTemplate(\"blank\");\r");
		strcat(buf,"\t\tp++;\r");
		strcat(buf,"\t\ti=0.5;\r");
		strcat(buf,"\t}\r\r");

		// Create 'name'  field 
		strcat(buf,"\tvar aRect = this.getPageBox( {nPage: p} );\r");
		strcat(buf,"\taRect[0] += 0.3*inch;\r");       
		strcat(buf,"\taRect[2] = aRect[0]+2*inch;\r"); 
		strcat(buf,"\taRect[1] -= i * inch;\r"); 
		strcat(buf,"\taRect[3] = aRect[1] - 40;\r\r"); 

		strcat(buf,"\tvar f = this.addField(\"name1.\"+ j, \"text\", p, aRect );\r");
		strcat(buf,"\tf.delay = true;\r");
		strcat(buf,"\tf.borderStyle = border.s;\r");
		strcat(buf,"\tf.borderColor = color.magenta;");
		strcat(buf,"\tf.textSize = 16;\r"); 
		strcat(buf,"\tf.textColor = color.blue;\r");
		strcat(buf,"\tf.fillColor = color.yellow;\r");
		strcat(buf,"\tf.readonly = true;\r");
		strcat(buf,"\tf.delay = false;\r\r");
			  
		// Create 'phone'  field 
		strcat(buf,"\tvar aRect = this.getPageBox( {nPage: p} );\r");
		strcat(buf,"\taRect[0] += 2.4*inch;\r");   
		strcat(buf,"\taRect[2] = aRect[0]+1.6*inch;\r");
		strcat(buf,"\taRect[1] -= i * inch;\r"); 
		strcat(buf,"\taRect[3] = aRect[1] - 40;\r\r");
							    
		strcat(buf,"\tvar f = this.addField(\"phone.\"+ j, \"text\", p, aRect );\r");
		strcat(buf,"\tf.delay = true;\r");
		strcat(buf,"\tf.borderStyle = border.s;\r");
		strcat(buf,"\tf.borderColor = color.magenta;");
		strcat(buf,"\tf.textSize = 0;\r"); 
		strcat(buf,"\tf.textColor = color.blue;\r");
		strcat(buf,"\tf.fillColor = color.yellow;\r");
		strcat(buf,"\tf.readonly = true;\r");
		strcat(buf,"\tf.delay = false;\r\r");

		// Create 'email' field 
		strcat(buf,"\tvar aRect = this.getPageBox( {nPage: p} );\r");
		strcat(buf,"\taRect[0] += 4.1*inch;\r");
		strcat(buf,"\taRect[2] = aRect[0]+1.2*inch;\r");
		strcat(buf,"\taRect[1] -= i * inch;\r"); 
		strcat(buf,"\taRect[3] = aRect[1] - 40;\r\r");
					
		strcat(buf,"\tvar f = this.addField(\"email.\" + j, \"button\", p, aRect );\r");
		strcat(buf,"\tf.delay = true;\r");
		strcat(buf,"\tf.borderColor = color.magenta;");
		strcat(buf,"\tf.borderStyle = border.i;\r");
		strcat(buf,"\tf.textColor = color.blue;\r");
		strcat(buf,"\tf.fillColor = color.yellow;\r");
		strcat(buf,"\tf.textSize = 16;\r");
		strcat(buf,"\tf.delay = false;\r\r");
					
		// Create 'location' field 
		strcat(buf,"\tvar aRect = this.getPageBox( {nPage: p} );\r");
		strcat(buf,"\taRect[0] += 5.4*inch;\r");
		strcat(buf,"\taRect[2] = aRect[0]+1.7*inch;\r");
		strcat(buf,"\taRect[1] -= i * inch;\r"); 
		strcat(buf,"\taRect[3] = aRect[1] - 40;\r\r");
					
		strcat(buf,"\tvar f = this.addField(\"location.\"+ j , \"text\", p, aRect );\r");
		strcat(buf,"\tf.delay = true;\r");
		strcat(buf,"\tf.borderStyle = border.s;\r");
		strcat(buf,"\tf.borderColor = color.magenta;");
		strcat(buf,"\tf.textSize = 13;\r"); 
		strcat(buf,"\tf.textColor = color.blue;\r");
		strcat(buf,"\tf.fillColor = color.yellow;\r");
		strcat(buf,"\tf.readonly = true;\r");
		strcat(buf,"\tf.delay = false;\r\r");
					
		// Create 'office_num' field 
		strcat(buf,"\tvar aRect = this.getPageBox( {nPage: p} );\r");
		strcat(buf,"\taRect[0] += 7.2*inch;\r");
		strcat(buf,"\taRect[2] = aRect[0]+1*inch;\r");
		strcat(buf,"\taRect[1] -= i * inch;\r"); 
		strcat(buf,"\taRect[3] = aRect[1] - 40;\r\r");
					
		strcat(buf,"\tvar f = this.addField(\"office_num.\" + j, \"text\", p, aRect );\r");
		strcat(buf,"\tf.delay = true;\r");
		strcat(buf,"\tf.borderStyle = border.s;\r");
		strcat(buf,"\tf.borderColor = color.magenta;");
		strcat(buf,"\tf.textSize = 14;\r");
		strcat(buf,"\tf.textColor = color.blue;\r");
		strcat(buf,"\tf.fillColor = color.yellow;\r");
		strcat(buf,"\tf.readonly = true;\r");
		strcat(buf,"\tf.delay = false;\r\r");
					
		// Create 'job_title' field 		
		strcat(buf,"\tvar aRect = this.getPageBox( {nPage: p} );\r");		
		strcat(buf,"\taRect[0] += 8.3*inch;\r");
		strcat(buf,"\taRect[2] = aRect[0]+2.5*inch;\r"); 
		strcat(buf,"\taRect[1] -= i * inch;\r"); 
		strcat(buf,"\taRect[3] = aRect[1] - 40;\r\r");
					
		strcat(buf,"\tvar f = this.addField(\"job_title.\" + j, \"text\", p, aRect );\r");		
		strcat(buf,"\tf.delay = true;\r");		
		strcat(buf,"\tf.borderStyle = border.s;\r");
		strcat(buf,"\tf.borderColor = color.magenta;");
		strcat(buf,"\tf.textSize = 0;\r"); 
		strcat(buf,"\tf.textColor = color.blue;\r");		
		strcat(buf,"\tf.fillColor = color.yellow;\r");		
		strcat(buf,"\tf.readonly = true;\r");		
		strcat(buf,"\tf.multiline = true;\r");	
		strcat(buf,"\tf.delay = false;\r\r");		
					
		// Create 'delimiter' field 
		strcat(buf,"\tvar aRect = this.getPageBox( {nPage: p} );\r");
		strcat(buf,"\taRect[0] += 0.3*inch;\r");       
		strcat(buf,"\taRect[2] = aRect[0]+10.5*inch;\r");
		strcat(buf,"\taRect[1] -=( i+0.6 ) * inch;\r");
		strcat(buf,"\taRect[3] = aRect[1] - 13;\r\r"); 
					
		strcat(buf,"\tvar f = this.addField(\"delimiter.\" + j, \"button\", p, aRect );\r");
		strcat(buf,"\tf.delay = true;\r");
		strcat(buf,"\tf.borderStyle = border.s;\r");
		strcat(buf,"\tf.borderColor = color.blue;");
		strcat(buf,"\tf.fillColor = color.ltGray;\r");
		strcat(buf,"\tf.readonly = true;\r");
		strcat(buf,"\tf.delay = false;\r");
		strcat(buf,"\ti+=0.8;\r");
		strcat(buf,"}\r");
			
			  
		theErc = FDFAddDocJavaScript(outFDF, "DrawOneRow" , buf );
		HandleError(theErc, "FDFAddDocJavaScript", "none", outFDF);

			  
		/*
		  Generate Document Level JavaScript function 'DrawResultField()'
		  which is responsible for drawing 'result' on the last page of
		  returned PDF document.
		*/
		strcpy(buf,"function DrawResultField()\r");
		strcat(buf,"{\r");
		strcat(buf,"\tvar inch=72;\r");	//??
		strcat(buf,"\tif( j%10 == 0 && j )\r");
		strcat(buf,"\t{\r");
		strcat(buf,"\t\tthis.spawnPageFromTemplate(\"blank\");\r");
		strcat(buf,"\t\tp++;\r");
		strcat(buf,"\t\ti=0.5;\r");
		strcat(buf,"\t};\r\r");
			  
		strcat(buf,"\tvar aRect = this.getPageBox( {nPage: p} );\r");
		strcat(buf,"\taRect[0] += 0.3*inch;\r");       
		strcat(buf,"\taRect[2] = aRect[0]+4*inch;\r");
		strcat(buf,"\taRect[1] -= (i+0.1)  * inch;\r");
		strcat(buf,"\taRect[3] = aRect[1] - 35;\r\r"); 
					
		strcat(buf,"\tvar f = this.addField(\"result\", \"text\", p, aRect );\r");
		strcat(buf,"\tf.delay = true;\r");
		strcat(buf,"\tf.borderStyle = border.b;\r");
		strcat(buf,"\tf.fillColor = color.cyan;\r");
		strcat(buf,"\tf.textColor = color.white;\r");
		strcat(buf,"\tf.borderColor = color.blue;\r");
		strcat(buf,"\tf.readonly = true;\r");
		strcat(buf,"\tf.delay = false;\r");
		strcat(buf,"}\r");
			  
		theErc = FDFAddDocJavaScript(outFDF, "DrawResultField" , buf );
		HandleError(theErc, "FDFAddDocJavaScript", "none", outFDF); 
			  
		/*
		  Generate Document Level JavaScript function 'SetCaption()'
		  This function sets caption for e-mail fields. 
		*/
			  
		strcpy(buf,"function SetCaption( string )\r");
		strcat(buf,"{\r");
		strcat(buf,"\taList = string.split(\",\");\r"); //string to array	
		strcat(buf,"\tfor( var i=0 ; i < aList.length-1; i++ ){\r");
		strcat(buf,"\t\tvar f = this.getField(\"email.\"+i);\r");
		strcat(buf,"\t\tf.buttonSetCaption(aList[i]);\r");
		strcat(buf,"\t}\r");	
		strcat(buf,"}\r");	
			  
		theErc = FDFAddDocJavaScript(outFDF, "SetCaption" , buf );
		HandleError(theErc, "FDFAddDocJavaScript", "none", outFDF); 
			  
		/*
		  Call generated Document Level functions using JavaScript
		  which get executed on Import event.
		*/
		char tmp1[25];
			  
		sprintf(tmp1, "for( j=0; j<%d; j++ )",count);
		strcpy(buf,tmp1);
		strcat(buf,"DrawOneRow();");
			  				  
		char* tmp2 = new char[ strlen(strEmail) + 20 ];
		sprintf(tmp2, "SetCaption(\" %s \");",strEmail);
		strcat(buf,tmp2);
		delete [] tmp2;
			  
		strcat(buf,"DrawResultField();");
			  
		theErc = FDFSetOnImportJavaScript(outFDF, buf , true);
		HandleError(theErc, "FDFSetOnImportJavaScript", "none", outFDF); 
			  
		/* Since the initial PDF that you submit from (PhnDirSrch.pdf) lives in
		   the same directory as SearchResult.pdf, it is OK to use a relative URL
		   for the FDFSetFile call */
		theErc = FDFSetFile( outFDF, "SearchResult.pdf");
		HandleError(theErc, "FDFSetFile", "none", outFDF);
	    }  
	    else { // if no records are found
		theErc = FDFSetStatus(outFDF, "No records were found. Please try again..." );
		HandleError(theErc, "FDFSetStatus", "none", outFDF);
	    }		  
	}
    }
    printf("Content-type: application/vnd.fdf\n\n");
	
    /*
      Don't forget to emit the correct HTTP header for the MIME type.
      By the way, remember when setting the URL of a SubmitForm action, to end
      it in #FDF. 
    */
    fflush(stdout);/* Emit the HTTP header now! */

    FDFSave(outFDF, "-"); /* Now send the FDF to stdout */
	
    FDFClose(outFDF);
    FDFClose(inFDF);

#ifdef UNIX_ENV
    /*
      This example assumes a cgi-bin environment, where each invocation of
      the program spawns a new process. In a multi-threaded environment,
      FDFFinalize() should only be called once, by the last thread in the
      application. Once again, this is Unix only. On Win32 the initialization
      and multithreading stuff is taken care of automatically by the FdfTk.dll
    */
    FDFFinalize();
#endif

    exit(0);
}

